from Foundation import *
from AppKit import *
import objc

# This module exists to provide a more Pythonic interface to the NSAlert class.

class AlertCallbackHandler(NSObject):
	
	def __init__(self, func):
		self.func = func
	
	@objc.signature("v@:@ii")
	def sheetDidEnd_returnCode_contextInfo_(cls, sheet, code, info):
		sheet.orderOut_(self)
		self.func(code)

def show_alert(window,
			   message,
			   text, 
			   buttons=["OK"],
			   callback=None):
	"""show_alert(window, message, text, buttons=["OK"], callback=None) -> None\n
Shows a modal NSAlert panel with the given window as its parent. show_alert() does not block.
'message' is the the title of the alert and 'text' is the body of the alert. 'buttons' is a list of
strings for the titles of the buttons in the alert. If 'callback' is not None, then it should be a
callable object that will be called when the alert finishes. The argument to 'callback' will be the
return code of the panel."""

	a = NSAlert.alloc().init()
	
	a.setMessageText_(message)
	
	a.setInformativeText_(text)
	
	for button in buttons:
		a.addButtonWithTitle_(button)
	
	if callback is not None:
		modalDelegate = AlertCallbackHandler(callback)
		didEndSelector = modalDelegate.sheetDidEnd_returnCode_contextInfo_
		contextInfo = 0
	else:
		modalDelegate = didEndSelector = None
		contextInfo = 0
	
	a.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo_(
		window, modalDelegate, didEndSelector, contextInfo)