import objc
from Foundation import *
from AppKit import *

from CE.Component import Component
from CE.Wire import Wire
from geometry import *

class SelectDragSession(object):
	"""SelectDragSession is used when the user is performing a rubber-band selection."""

	def __init__(self, view, start):
		self.view = view
		self.start = self.end = start
		self.selection = set()
		self.make_rect()
	
	def draw(self):
		for obj in self.selection:
			obj.draw_selection()
		
		NSColor.blueColor().set()
		NSBezierPath.strokeRect_(self.rect)
	
	def mouse_dragged(self, point):
		self.redraw_rect()
		
		old_selection = self.selection.copy()
		self.end = point
		self.make_rect()
		
		for obj in old_selection.symmetric_difference(self.selection):
			if isinstance(obj, Component):
				self.view.setNeedsDisplayInRect_(expand_rect(obj.bounds(), 1))
			else:
				self.view.setNeedsDisplayInRect_(rect_around_point(obj.location, 1))
		
		self.redraw_rect()
	
	def mouse_up(self, point):
		self.view.selection = self.view.selection.union(self.selection)
		self.view.setNeedsDisplay_(True)
	
	def redraw_rect(self):
		left = self.rect[0][0]
		bottom = self.rect[0][1]
		width = self.rect[1][0]
		height = self.rect[1][1]
		right = left + width
		top = bottom + height
		self.view.setNeedsDisplayInRect_( ((left-0.1,bottom-0.1),  (width+0.2,0.2))  )
		self.view.setNeedsDisplayInRect_( ((left-0.1,bottom-0.1),  (0.2,height+0.2)) )
		self.view.setNeedsDisplayInRect_( ((left-0.1,top-0.1),     (width+0.2,0.2))  )
		self.view.setNeedsDisplayInRect_( ((right-0.1,bottom-0.1), (0.2,height+0.2)) )
	
	def make_rect(self):
		left = min(self.start[0], self.end[0])
		right = max(self.start[0], self.end[0])
		bottom = min(self.start[1], self.end[1])
		top = max(self.start[1], self.end[1])
		self.rect = ((left, bottom), (right-left, top-bottom))
				
		self.selection = set()
		for obj in self.view.circuit.components:
			if rects_intersect(obj.bounds(), self.rect):
				self.selection.add(obj)
		for wire in self.view.circuit.wires:
			for obj in (wire.start, wire.end):
				if point_in_rect(obj.location, self.rect):
					self.selection.add(obj)