from __future__ import division

from Foundation import *
from AppKit import *
import objc
from PyObjCTools import NibClassBuilder

# Local imports from within CircuitEditor
from geometry import *
import circuit_serialize
import circuit_photograph
from CE import component_classes

class CircuitPalette(NibClassBuilder.AutoBaseClass):
	
	# The following outlets are defined in the nib:
	# "paletteView"
	
	def init(self):
		self.initWithWindowNibName_("CircuitPalette")
		self.showWindow_(self)
		
		return self
	
	def windowDidLoad(self):
		assert(self.paletteView)
		width = self.paletteView.frame().size.width
		height = 0
		
		for component_class in reversed(component_classes):
		
			prototype = component_class()
			
			item_height = prototype.height*16+30
			frame = ((0,height), (width,item_height))
			bounds = (
				((prototype.width-width/16)/2, (prototype.height-item_height/16)/2),
				(width/16, item_height/16))
				
			palette_item = CircuitPaletteItem.alloc().initWithFrame_(frame)
			palette_item.setBounds_(bounds)
			palette_item.prototype = prototype
			
			self.paletteView.addSubview_(palette_item)
			height += item_height
		
		self.paletteView.setFrameSize_((width,height))
		
		clipView = self.paletteView.superview()
		clipView.scrollToPoint_(clipView.constrainScrollPoint_((0, height)))
		clipView.superview().reflectScrolledClipView_(clipView)
		

class CircuitPaletteItem(NSView):

	def drawRect_(self, rect):
		NSColor.whiteColor().set()
		NSRectFill(self.bounds())
		self.prototype.draw()

	def acceptsFirstMouse_(self, event):
		return True

	def mouseDragged_(self, event):
	
		# Create a picture representing the dragged object
		bounds = expand_rect(self.prototype.bounds(), 1)
		image = NSImage.alloc().initWithSize_(scale_point(bounds[1], 16.0))
		image.addRepresentation_(
				circuit_photograph.photograph(bounds, set([self.prototype]), []))
		
		# Prepare the pasteboard
		pboard = NSPasteboard.pasteboardWithName_(NSDragPboard)
		pboard.declareTypes_owner_(["CircuitComponents"], None)
		serialized = circuit_serialize.serialize_elements([self.prototype])
		pboard.setString_forType_(serialized, "CircuitComponents")
		
		# This call blocks until the drag is complete
		self.dragImage_at_offset_event_pasteboard_source_slideBack_(
			image,
			bounds[0],
			NSZeroSize, # Ignored
			event,
			pboard,
			self,
			False)