import CE
from CE.Component import Component
from CE.Wire import Wire
import random

# There are two modes for serializing objects. If the objects are serialized with order preserved,
# then they will be loaded in the same order they were saved. If they are serialized without order
# preserved, they will be loaded in a randomized order each time. In theory, the order should not
# matter, but in practice, some bugs depend on the order in which the components are written. 
# Normally, the random-order mode is used for everything except for debugging. When a debug dump is 
# written to a file, the circuit being debugged is also written in order-preserving mode so that the 
# problem can be reproduced if necessary.

def serialize_elements(objs, preserve_order=False):
	# Serializes a set of Wire and Component objects into a single string. If preserve_order is 
	# true, then the order will be preserved. 
	l = []
	for obj in objs:
		l.append((obj.__class__.__name__, obj.serialize()))
	if preserve_order:
		return repr(l)
	else:
		return repr(set(l))

def deserialize_elements(string):
	# Converts a string returned by serialize_elements into a list or set of Wire and Component 
	# objects. 
	list_or_set = eval(string)
	objs = []
	for classname, info in list_or_set:
		objs.append(serializable_class_table[classname](info))
	
	if isinstance(list_or_set, list):
		return objs
	else:
		# If the object was recorded as a set, then shuffle the list so that lots of different
		# orderings get tried. This is a feature to help flush out bugs that only show up in certain
		# orderings.
		random.shuffle(objs)
		return objs

# In the serialized string, classes are identified by name. Build a table of classes...
serializable_classes = [Wire] + CE.component_classes
serializable_class_table = dict((c.__name__, c) for c in serializable_classes)